#! /usr/bin/python

"""
Just for fun, rewrote a script using php / ls / and awk at the same time ;)

Produces the following output:

toad@vlk:~/python$ ./perm.py ././azel/../azel/examples/gistit.py  
drwxr-xr-x root root /home
drwxr-xr-x toad toad /home/toad
drwxr-xr-x toad toad /home/toad/python
drwxr-xr-x toad toad /home/toad/python/azel
drwxr-xr-x toad toad /home/toad/python/azel/examples
-rwxr-xr-x toad toad /home/toad/python/azel/examples/gistit.py
"""

import sys
import os
import pwd
import grp
import stat
cwd = os.getcwd() + '/'
"""
# Get the target directory
if len(sys.argv) > 1:
    target = sys.argv[1]
else:
    target = cwd

# Generate the absolute path
if not target.startswith('/'):
    target = cwd + target
target = os.path.realpath(target)
"""

# Recursive (collect target directories / file)
def dirs(target, target_l):
    target_l.insert(0, target)
    d, t = os.path.split(target)
    if d is '/':
#    if target is '/': # if we didnt want to ignore /
        return target_l
    return dirs(d, target_l)

# Display the permission on a given target
def perm(target):
    s = os.lstat(target)
    # http://git.savannah.gnu.org/cgit/coreutils.git/tree/src/ls.c
    # http://docs.python.org/library/stat.html#module-stat
    mode = ''
    # Block type
    if stat.S_ISDIR(s.st_mode):
        mode = 'd'
    elif stat.S_ISBLK(s.st_mode):
        mode = 'b'
    elif stat.S_ISLNK(s.st_mode):
        mode = 'l'
    elif stat.S_ISFIFO(s.st_mode):
        mode = 'p'
    elif stat.S_ISCHR(s.st_mode):
        mode = 'c'
    elif stat.S_ISSOCK(s.st_mode):
        mode = 's'
    else:
        mode = '-'

    # User
    if (stat.S_IRUSR & s.st_mode):
        mode += 'r'
    else:
        mode += '-'
    if (stat.S_IWUSR & s.st_mode):
        mode += 'w'
    else:
        mode += '-'
    if (stat.S_ISUID & s.st_mode):
        mode += 's'
    elif (stat.S_IXUSR & s.st_mode):
        mode += 'x'
    else:
        mode += '-'

    # Group
    if (stat.S_IRGRP & s.st_mode):
        mode += 'r'
    else:
        mode += '-'
    if (stat.S_IWGRP & s.st_mode):
        mode += 'w'
    else:
        mode += '-'
    if (stat.S_ISGID & s.st_mode):
        mode += 's'
    elif (stat.S_IXGRP & s.st_mode):
        mode += 'x'
    else:
        mode += '-'

    # Others
    if (stat.S_IROTH & s.st_mode):
        mode += 'r'
    else:
        mode += '-'
    if (stat.S_IWOTH & s.st_mode):
        mode += 'w'
    else:
        mode += '-'

    if (stat.S_ISVTX & s.st_mode):
        mode += 't'
    elif (stat.S_IXOTH & s.st_mode):
        mode += 'x'
    else:
        mode += '-'

    try:
        uname = pwd.getpwuid(s.st_uid).pw_name
    except KeyError:
        uname = s.st_uid
    try:
        gname = grp.getgrgid(s.st_gid).gr_name
    except KeyError:
        gname = s.st_gid
    print "%s \t%s \t%s \t%s" % (mode, uname, gname, target)

# Get the target directory
# If no parameter was passed use the current working directory
if len(sys.argv) == 1:
    sys.argv.append(cwd)

done = {} # avoid showing the same directories
for i in range(1, len(sys.argv)):
    target = sys.argv[i]

    # Generate the absolute path
    if not target.startswith('/'):
        target = cwd + target
    target = os.path.realpath(target)

    target_l = dirs(target, [])
    for target in target_l:
        if target in done:
            continue
        perm(target)
        done[target] = True

